(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.1 1998/07/28 13:02:10 xleroy Exp $ *)

(* The translator from from 1.07 syntax to 2.00 syntax *)

open Format
open Location

let to_stdout = ref false
let rename_old = ref false

let parse_file inputfile parse_fun transl_fun =
  let ic = open_in_bin inputfile in
  let ast =
    try
      Location.input_name := inputfile;
      parse_fun (Lexing.from_channel ic)
    with x ->
      close_in ic; raise x in
  seek_in ic 0;
  let oc =
    if !to_stdout then stdout else open_out_bin (inputfile ^ ".new") in
  try
    transl_fun ic oc ast;
    close_in ic;
    close_out oc;
    if !rename_old then begin
      Sys.rename inputfile (inputfile ^ ".old");
      Sys.rename (inputfile ^ ".new") inputfile
    end
  with x ->
    close_in ic; close_out oc; Misc.remove_file (inputfile ^ ".new"); raise x

let process_file f =
  if Filename.check_suffix f ".ml" then
    parse_file f Parse.implementation Translate.transl_implementation
  else if Filename.check_suffix f ".mli" then
    parse_file f Parse.interface Translate.transl_interface
  else
    raise(Arg.Bad("don't know what to do with " ^ f))

let main () =
  try
    Arg.parse [
      "-print", Arg.Set to_stdout,
        "print result on stdout (default is save in .new file)";
      "-replace", Arg.Set rename_old,
        "replace source file by result of conversion, save original in .old file"
    ] process_file
    "Usage: ocaml1to2 [options] <source files>\nOptions are:"
  with x ->
    Format.set_formatter_out_channel stderr;
    open_box 0;
    begin match x with
      Lexer.Error(err, start, stop) ->
        Location.print {loc_start = start; loc_end = stop};
        Lexer.report_error err
    | Syntaxerr.Error err ->
        Syntaxerr.report_error err
    | x ->
        close_box(); raise x
    end;
    close_box(); print_newline(); exit 2

let _ = main()

